@@ -101,11 +101,19 @@ pub struct EguiSettings {
     /// }
     /// ```
     pub scale_factor: f64,
+    /// Will be used as a default value for hyperlink [target](https://www.w3schools.com/tags/att_a_target.asp) hints.
+    /// If not specified, `_self` will be used. Only matters in a web browser.
+    #[cfg(feature = "open_url")]
+    pub default_open_url_target: Option<String>,
 }
 
 impl Default for EguiSettings {
     fn default() -> Self {
-        Self { scale_factor: 1.0 }
+        Self {
+            scale_factor: 1.0,
+            #[cfg(feature = "open_url")]
+            default_open_url_target: None,
+        }
     }
 }
 
@@ -230,9 +238,9 @@ impl EguiContext {
 
     /// Egui context of the primary window.
     ///
-    /// This function is only available when the `multi_threaded` feature is enabled.
+    /// This function is only available when the `immutable_ctx` feature is enabled.
     /// The preferable way is to use `ctx_mut` to avoid unpredictable blocking inside UI systems.
-    #[cfg(feature = "multi_threaded")]
+    #[cfg(feature = "immutable_ctx")]
     #[must_use]
     #[track_caller]
     pub fn ctx(&self) -> &egui::Context {
@@ -243,10 +251,10 @@ impl EguiContext {
     /// If you want to display UI on a non-primary window, make sure to set up the render graph by
     /// calling [`setup_pipeline`].
     ///
-    /// This function is only available when the `multi_threaded` feature is enabled.
+    /// This function is only available when the `immutable_ctx` feature is enabled.
     /// The preferable way is to use `ctx_for_window_mut` to avoid unpredictable blocking inside UI
     /// systems.
-    #[cfg(feature = "multi_threaded")]
+    #[cfg(feature = "immutable_ctx")]
     #[must_use]
     #[track_caller]
     pub fn ctx_for_window(&self, window: WindowId) -> &egui::Context {
@@ -258,10 +266,10 @@ impl EguiContext {
     /// Fallible variant of [`EguiContext::ctx_for_window`]. Make sure to set up the render graph by
     /// calling [`setup_pipeline`].
     ///
-    /// This function is only available when the `multi_threaded` feature is enabled.
+    /// This function is only available when the `immutable_ctx` feature is enabled.
     /// The preferable way is to use `try_ctx_for_window_mut` to avoid unpredictable blocking inside
     /// UI systems.
-    #[cfg(feature = "multi_threaded")]
+    #[cfg(feature = "immutable_ctx")]
     #[must_use]
     pub fn try_ctx_for_window(&self, window: WindowId) -> Option<&egui::Context> {
         self.ctx.get(&window)
@@ -293,7 +301,7 @@ impl EguiContext {
     }
 
     /// Allows to get multiple contexts at the same time. This function is useful when you want
-    /// to get multiple window contexts without using the `multi_threaded` feature.
+    /// to get multiple window contexts without using the `immutable_ctx` feature.
     ///
     /// # Panics
     ///
@@ -463,7 +471,6 @@ impl Plugin for EguiPlugin {
         app.add_system_to_stage(CoreStage::Last, free_egui_textures);
 
         if let Ok(render_app) = app.get_sub_app_mut(RenderApp) {
-            render_app.init_resource::<egui_node::EguiPipeline>();
             render_app
                 .init_resource::<egui_node::EguiPipeline>()
                 .init_resource::<EguiTransforms>()
